'''
(c) 2017 DigitalOcean

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''
import netaddr
import six

from pynetbox.lib.query import Request


def get_return(lookup, return_fields=None):

    for i in return_fields or ['id', 'value', 'nested_return']:
        if isinstance(lookup, dict) and lookup.get(i):
            return lookup[i]
        else:
            if hasattr(lookup, i):
                return getattr(lookup, i)
    if isinstance(lookup, Record):
        return str(lookup)
    else:
        return lookup


def flatten_custom(custom_dict):
    return {
        k: v if not isinstance(v, dict) else v['value']
        for k, v in custom_dict.items()
    }


class Record(object):
    """Create python objects from netbox API responses.

        Iterates over `values` and checks if the `key` is defined as a
        a class attribute point to another Record object. If so, it
        then takes `value` and instantiates the referenced object
        creating a nested object of whatever type is requested in the
        class attribute.

        If `key` doesn't match a class attribute an instance attribute
        is set containing `value`.

        :arg dict values: The response from the netbox api.
        :arg dict api_kwargs: Contains the arguments passed to Api()
            when it was instantiated.
    """

    url = None
    has_details = False

    def __init__(self, values, api_kwargs={}, endpoint_meta={}):
        self._full_cache = []
        self._index_cache = []
        self.api_kwargs = api_kwargs
        self.endpoint_meta = endpoint_meta
        self.default_ret = Record

        if values:
            self._parse_values(values)

    def __getattr__(self, k):
        """Default behavior for missing attrs.

        We'll call `full_details()` if we're asked for an attribute
        we don't have.

        In order to prevent non-explicit behavior,`k='keys'` is
        excluded because casting to dict() calls this attr.
        """
        if self.url:
            if self.has_details is False and k != 'keys':
                if self.full_details():
                    ret = getattr(self, k, None)
                    if ret or hasattr(self, k):
                        return ret

        raise AttributeError('object has no attribute "{}"'.format(k))

    def __iter__(self):
        for i in dict(self._full_cache).keys():
            cur_attr = getattr(self, i)
            if isinstance(cur_attr, Record):
                yield i, dict(cur_attr)
            else:
                yield i, cur_attr

    def __getitem__(self, item):
        return item

    def __str__(self):
        return (
            getattr(self, 'name', None) or
            getattr(self, 'label', None) or
            ''
        )

    def __repr__(self):
        return str(self)

    def __getstate__(self):
        return self.__dict__

    def __setstate__(self, d):
        self.__dict__.update(d)

    def _add_cache(self, item):
        key, value = item
        if isinstance(value, Record):
            self._full_cache.append((key, dict(value)))
        else:
            self._full_cache.append((key, value))
        self._index_cache.append((key, get_return(value)))

    def _parse_values(self, values):
        """ Parses values init arg.

        Parses values dict at init and sets object attributes with the
        values within.
        """
        for k, v in values.items():
            if k != 'custom_fields':
                if isinstance(v, dict):
                    lookup = getattr(self.__class__, k, None)
                    if lookup:
                        v = lookup(v, api_kwargs=self.api_kwargs)
                    else:
                        v = self.default_ret(v, api_kwargs=self.api_kwargs)
                self._add_cache((k, v))
            else:
                self._add_cache((k, v.copy()))
            setattr(self, k, v)

    def _compare(self):
        """Compares current attributes to values at instantiation.

        In order to be idempotent we run this method in `save()`.

        Returns:
            Boolean value, True indicates current instance has the same
            attributes as the ones passed to `values`.
        """
        init_dict = {}
        init_vals = dict(self._index_cache)
        for i in dict(self):
            current_val = init_vals.get(i)
            if i == 'custom_fields':
                init_dict[i] = flatten_custom(current_val)
            else:
                init_dict[i] = get_return(current_val)

        if init_dict == self.serialize():
            return True
        return False

    def full_details(self):
        """Queries the hyperlinked endpoint if 'url' is defined.

        This method will populate the attributes from the detail
        endpoint when it's called. Sets the class-level `has_details`
        attribute when it's called to prevent being called more
        than once.

        :returns: True
        """
        if self.url:
            req = Request(
                base=self.url,
                token=self.api_kwargs.get('token'),
                session_key=self.api_kwargs.get('session_key'),
                version=self.api_kwargs.get('version'),
                ssl_verify=self.api_kwargs.get('ssl_verify')
            )
            self._parse_values(req.get())
            self.has_details = True
            return True
        return False

    def serialize(self, nested=False):
        """Serializes an object

        Pulls all the attributes in an object and creates a dict that
        can be turned into the json that netbox is expecting.

        If an attribute's value is a ``Record`` or ``IPRecord`` type
        it's replaced with the ``id`` field of that object.

        :returns: dict of values the NetBox API is expecting.
        """
        if nested:
            return get_return(self)

        ret = {}
        for i in dict(self):
            current_val = getattr(self, i)
            if i == 'custom_fields':
                ret[i] = flatten_custom(current_val)
            else:
                if isinstance(current_val, Record):
                    current_val = getattr(
                        current_val,
                        'serialize'
                    )(nested=True)

                if isinstance(current_val, netaddr.ip.IPNetwork):
                    current_val = str(current_val)

                ret[i] = current_val
        return ret

    def save(self):
        """Saves changes to an existing object.

        Runs self.serialize() and checks that it doesn't match
        self._compare(). If not create a Request object and run .put()

        :returns: True if PUT request was successful.
        :example:

        >>> x = nb.dcim.devices.get(name='test1-a3-tor1b')
        >>> x.serial
        u''
        >>> x.serial = '1234'
        >>> x.save()
        True
        >>>
        """
        if self.id:
            if not self._compare():
                req = Request(
                    key=self.id,
                    base=self.endpoint_meta.get('url'),
                    token=self.api_kwargs.get('token'),
                    session_key=self.api_kwargs.get('session_key'),
                    version=self.api_kwargs.get('version'),
                    ssl_verify=self.api_kwargs.get('ssl_verify')
                )
                if req.put(self.serialize()):
                    return True
            else:
                return False

    def delete(self):
        """Deletes an existing object.

        :returns: True if DELETE operation was successful.
        :example:

        >>> x = nb.dcim.devices.get(name='test1-a3-tor1b')
        >>> x.delete()
        True
        >>>
        """
        req = Request(
            key=self.id,
            base=self.endpoint_meta.get('url'),
            token=self.api_kwargs.get('token'),
            session_key=self.api_kwargs.get('session_key'),
            version=self.api_kwargs.get('version'),
            ssl_verify=self.api_kwargs.get('ssl_verify')
        )
        if req.delete():
            return True
        else:
            return False


class IPRecord(Record):
    """IP-specific Record for IPAM responses.

    Extends ``Record`` objects to handle replacing ip4/6 strings with
    instances of ``netaddr.IPNetworks`` instead.
    """

    def __init__(self, *args, **kwargs):
        super(IPRecord, self).__init__(*args, **kwargs)
        self.default_ret = IPRecord

    def __iter__(self):
        for i in dict(self._full_cache).keys():
            cur_attr = getattr(self, i)
            if isinstance(cur_attr, Record):
                yield i, dict(cur_attr)
            else:
                if isinstance(cur_attr, netaddr.IPNetwork):
                    yield i, str(cur_attr)
                else:
                    yield i, cur_attr

    def _parse_values(self, values):
        """ Parses values init arg. for responses with IPs fields.

        Similar parser as parent, but takes str & unicode fields and
        trys converting them to IPNetwork objects.
        """
        for k, v in values.items():
            if k != 'custom_fields':
                if isinstance(v, dict):
                    lookup = getattr(self.__class__, k, None)
                    if lookup:
                        v = lookup(v, api_kwargs=self.api_kwargs)
                    else:
                        v = self.default_ret(v, api_kwargs=self.api_kwargs)
                if isinstance(v, six.string_types):
                    try:
                        v = netaddr.IPNetwork(v)
                    except netaddr.AddrFormatError:
                        pass
                self._add_cache((k, v))
            else:
                self._add_cache((k, v.copy()))
            setattr(self, k, v)



import unittest
import json

import netaddr
import six

from .util import Response
import pynetbox

if six.PY3:
    from unittest.mock import patch
else:
    from mock import patch


api = pynetbox.api(
    "http://localhost:8000",
    version='2.0'
)

nb = api.ipam

HEADERS = {
    'accept': 'application/json; version=2.0;'
}

POST_HEADERS = {
    'Content-Type': 'application/json; version=2.0;',
    'authorization': 'Token None',
}


class GenericTest(object):
    name = None
    name_singular = None
    ret = pynetbox.lib.response.IPRecord
    app = 'ipam'
    ip_obj_fields = {}

    def test_get_all(self):
        with patch(
            'pynetbox.lib.query.requests.get',
            return_value=Response(fixture='{}/{}.json'.format(
                self.app,
                self.name
            ))
        ) as mock:
            ret = getattr(nb, self.name).all()
            self.assertTrue(ret)
            self.assertTrue(isinstance(ret, list))
            self.assertTrue(isinstance(ret[0], self.ret))
            mock.assert_called_with(
                'http://localhost:8000/api/{}/{}/'.format(
                    self.app,
                    self.name.replace('_', '-')
                ),
                headers=HEADERS,
                verify=True
            )

    def test_filter(self):
        with patch(
            'pynetbox.lib.query.requests.get',
            return_value=Response(fixture='{}/{}.json'.format(
                self.app,
                self.name
            ))
        ) as mock:
            ret = getattr(nb, self.name).filter(pk=1)
            self.assertTrue(ret)
            self.assertTrue(isinstance(ret, list))
            self.assertTrue(isinstance(ret[0], self.ret))
            mock.assert_called_with(
                'http://localhost:8000/api/{}/{}/?pk=1'.format(
                    self.app,
                    self.name.replace('_', '-')
                ),
                headers=HEADERS,
                verify=True
            )

    def test_get(self):
        with patch(
            'pynetbox.lib.query.requests.get',
            return_value=Response(fixture='{}/{}.json'.format(
                self.app,
                self.name_singular or self.name[:-1]
            ))
        ) as mock:
            ret = getattr(nb, self.name).get(1)
            self.assertTrue(ret)
            self.assertTrue(isinstance(ret, self.ret))
            self.assertTrue(isinstance(dict(ret), dict))
            self.assertTrue(isinstance(str(ret), str))
            mock.assert_called_with(
                'http://localhost:8000/api/{}/{}/1/'.format(
                    self.app,
                    self.name.replace('_', '-')
                ),
                headers=HEADERS,
                verify=True
            )
            if self.ip_obj_fields:
                for field in self.ip_obj_fields:
                    self.assertTrue(
                        isinstance(getattr(ret, field), netaddr.IPNetwork)
                    )
                    self.assertTrue(netaddr.IPNetwork(dict(ret)[field]))


class PrefixTestCase(unittest.TestCase, GenericTest):
    name = 'prefixes'
    name_singular = 'prefix'
    ip_obj_fields = ['prefix']

    @patch(
        'pynetbox.lib.query.requests.get',
        return_value=Response(fixture='ipam/prefix.json')
    )
    def test_modify(self, mock):
        ret = nb.prefixes.get(1)
        ret.prefix = '10.1.2.0/24'
        ret_serialized = ret.serialize()
        self.assertTrue(ret_serialized)
        self.assertFalse(ret._compare())
        self.assertEqual(ret_serialized['prefix'], '10.1.2.0/24')
        self.assertTrue(netaddr.IPNetwork(ret_serialized['prefix']))

    @patch(
        'pynetbox.lib.query.requests.get',
        side_effect=[
            Response(fixture='ipam/prefix.json'),
            Response(fixture='ipam/available-ips.json'),
        ]
    )
    def test_get_available_ips(self, mock):
        pfx = nb.prefixes.get(1)
        ret = pfx.available_ips.list()
        mock.assert_called_with(
            'http://localhost:8000/api/ipam/prefixes/1/available-ips/'.format(self.name),
            headers=HEADERS,
            verify=True
        )
        self.assertTrue(ret)
        self.assertEqual(len(ret), 3)

    @patch(
        'pynetbox.lib.query.requests.post',
        return_value=Response(fixture='ipam/available-ips-post.json')
    )
    @patch(
        'pynetbox.lib.query.requests.get',
        return_value=Response(fixture='ipam/prefix.json'),
    )
    def test_create_available_ips(self, get, post):
        expected_result = {
            'status': 1,
            'description': '',
            'nat_inside': None,
            'role': None,
            'vrf': None,
            'address':
            '10.1.1.1/32',
            'interface': None,
            'id': 1,
            'tenant': None
        }
        create_parms = dict(
            status=2,
        )
        pfx = nb.prefixes.get(1)
        ret = pfx.available_ips.create(create_parms)
        post.assert_called_with(
            'http://localhost:8000/api/ipam/prefixes/1/available-ips/'.format(self.name),
            headers=POST_HEADERS,
            data=json.dumps(create_parms),
            verify=True
        )
        self.assertTrue(ret)
        self.assertEqual(ret, expected_result)

    @patch(
        'pynetbox.lib.query.requests.get',
        side_effect=[
            Response(fixture='ipam/prefix.json'),
            Response(fixture='ipam/available-prefixes.json'),
        ]
    )
    def test_get_available_prefixes(self, mock):
        pfx = nb.prefixes.get(1)
        ret = pfx.available_prefixes.list()
        mock.assert_called_with(
            'http://localhost:8000/api/ipam/prefixes/1/available-prefixes/'.format(self.name),
            headers=HEADERS,
            verify=True
        )
        self.assertTrue(ret)

    @patch(
        'pynetbox.lib.query.requests.post',
        return_value=Response(fixture='ipam/available-prefixes-post.json')
    )
    @patch(
        'pynetbox.lib.query.requests.get',
        return_value=Response(fixture='ipam/prefix.json'),
    )
    def test_create_available_prefixes(self, get, post):
        create_parms = dict(
            prefix_length=30,
        )
        pfx = nb.prefixes.get(1)
        ret = pfx.available_prefixes.create(create_parms)
        post.assert_called_with(
            'http://localhost:8000/api/ipam/prefixes/1/available-prefixes/'.format(self.name),
            headers=POST_HEADERS,
            data=json.dumps(create_parms),
            verify=True
        )
        self.assertTrue(ret)


class IPAddressTestCase(unittest.TestCase, GenericTest):
    name = 'ip_addresses'
    name_singular = 'ip_address'
    ip_obj_fields = ['address']

    @patch(
        'pynetbox.lib.query.requests.get',
        return_value=Response(fixture='ipam/ip_address.json')
    )
    def test_modify(self, mock):
        ret = nb.prefixes.get(1)
        ret.description = 'testing'
        ret_serialized = ret.serialize()
        self.assertTrue(ret_serialized)
        self.assertFalse(ret._compare())
        self.assertEqual(ret_serialized['address'], '10.0.255.1/32')
        self.assertEqual(ret_serialized['description'], 'testing')
        self.assertTrue(netaddr.IPNetwork(ret_serialized['address']))


class RoleTestCase(unittest.TestCase, GenericTest):
    name = 'roles'


class RIRTestCase(unittest.TestCase, GenericTest):
    name = 'rirs'


class AggregatesTestCase(unittest.TestCase, GenericTest):
    name = 'aggregates'
    ip_obj_fields = ['prefix']


class VlanTestCase(unittest.TestCase, GenericTest):
    name = 'vlans'


class VlanGroupsTestCase(unittest.TestCase, GenericTest):
    name = 'vlan_groups'


class VRFTestCase(unittest.TestCase, GenericTest):
    name = 'vrfs'


# class ServicesTestCase(unittest.TestCase, GenericTest):
#     name = 'services'



'''
(c) 2017 DigitalOcean

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''
import json
from six.moves.urllib.parse import urlencode

import requests


def url_param_builder(param_dict):
    '''Builds url parameters

    Creates URL paramters (e.g. '.../?xyz=r21&abc=123') from a dict
    passed in param_dict
    '''
    return '?{}'.format(urlencode(param_dict))


class RequestError(Exception):
    """Basic Request Exception

    More detailed exception that returns the requests object. Along
    with some attributes with specific details from the requests
    object.

    :Example:

    >>> try:
    ...   nb.dcim.devices.create({'name': 'destined-for-failure'})
    ... except pynetbox.RequestError as e:
    ...   print(e.error)
    ...
    {"device_role":["This field is required."]}



    """
    def __init__(self, message):
        req = message
        message = "The request failed with code {} {}".format(
            req.status_code,
            req.reason
        )
        super(RequestError, self).__init__(message)
        self.req = req
        self.request_body = req.request.body
        self.url = req.url
        self.error = req.text


class Request(object):
    """Creates requests to the Netbox API

    Responsible for building the url and making the HTTP(S) requests to
    Netbox's API

    :param base: (str) Base URL passed in api() instantiation.
    :param filters: (dict, optional) contains key/value pairs that
        correlate to the filters a given endpoint accepts.
        In (e.g. /api/dcim/devices/?name='test') 'name': 'test'
        would be in the filters dict.
    :param private_key: (str, optional) The user's private key as a
        string.
    """

    def __init__(self, base=None, filters=None, key=None, token=None,
                 private_key=None, version=None, session_key=None,
                 ssl_verify=True):
        """
        Instantiates a new Request object

        Args:
            base (string): Base URL passed in api() instantiation.
            filters (dict, optional): contains key/value pairs that
                correlate to the filters a given endpoint accepts.
                In (e.g. /api/dcim/devices/?name='test') 'name': 'test'
                would be in the filters dict.
            key (int, optional): database id of the item being queried.
            private_key (string, optional): The user's private key as a
                string.
        """
        self.base = base
        self.filters = filters
        self.key = key
        self.token = token
        self.private_key = private_key
        self.version = version
        self.session_key = session_key
        self.ssl_verify = ssl_verify

    def get_version(self):
        """Query the netbox API for its API-Version.

        Queries the API root for the *API-Verion* header.

        :returns: String containing version.
        """
        ret = requests.get(
            "{}/".format(self.base),
            verify=self.ssl_verify
        ).headers.get('API-Version', '1.0')
        setattr(self, 'version', ret)
        return ret

    def get_session_key(self):
        """Requests session key

        Issues a GET request to the `get-session-key` endpoint for
        subsequent use in requests from the `secrets` endpoint.

        :Returns: String containing session key.
        """
        req = requests.post(
            '{}/secrets/get-session-key/?preserve_key=True'.format(self.base),
            headers={
                'accept': 'application/json',
                'authorization': 'Token {}'.format(self.token),
                'Content-Type': 'application/x-www-form-urlencoded',
            },
            data=urlencode({
                'private_key': self.private_key.strip('\n')
            }),
            verify=self.ssl_verify
        )
        if req.ok:
            return req.json()['session_key']
        else:
            raise RequestError(req)

    @property
    def url(self):
        """ Builds a URL.
        Creates a valid netbox url based on kwargs passed during
        instantiation.

        :Returns: String of URL.
        """
        def construct_url(input):
            for k, v in input.items():
                if isinstance(v, list):
                    for i in v:
                        yield urlencode({k: i})
                else:
                    yield urlencode({k: v})

        if self.key:
            return '{}/{key}/'.format(
                self.base,
                key=self.key
            )
        if self.filters:
            if self.base[-1] == '/':
                return '{}?{query}'.format(
                    self.base,
                    query="&".join(list(construct_url(self.filters)))
                )
            elif '/?' in self.base:
                return '{}&{query}'.format(
                    self.base,
                    query="&".join(list(construct_url(self.filters)))
                )
            else:
                return '{}/?{query}'.format(
                    self.base,
                    query="&".join(list(construct_url(self.filters)))
                )
        else:
            return self.base

    def normalize_url(self, url):
        """ Builds a url for POST actions.
        """
        if url[-1] != '/':
            return "{}/".format(url)

        return url

    def get(self):
        """Makes a GET request.

        Makes a GET request to NetBox's API, and automatically recurses
        any paginated results.

        :raises: RequestError if req.ok returns false.

        :Returns: List of `Response` objects returned from the
            endpoint.
        """
        headers = {
            'accept': 'application/json; version={};'.format(
                self.version or self.get_version()
            )
        }
        if self.token:
            headers.update(authorization='Token {}'.format(self.token))
        if self.session_key:
            headers.update(
                {'X-Session-Key': self.session_key}
            )

        def make_request(url):

            req = requests.get(url, headers=headers, verify=self.ssl_verify)
            if req.ok:
                return req.json()
            else:
                raise RequestError(req)

        def req_all(url):
            req = make_request(url)
            if isinstance(req, dict) and req.get('results') is not None:
                ret = req['results']
                first_run = True
                while req['next']:
                    next_url = "{}{}limit={}&offset={}".format(
                        self.url,
                        '&' if self.url[-1] != '/' else '?',
                        req['count'],
                        len(req['results'])
                    ) if first_run else req['next']
                    req = make_request(next_url)
                    first_run = False
                    ret.extend(req['results'])
                return ret
            else:
                return req

        return req_all(self.url)

    def put(self, data):
        """Makes PUT request.

        Makes a PUT request to NetBox's API. Adds the session key to
        headers if the `private_key` attribute was populated.

        :param data: (dict) Contains a dict that will be turned into a
            json object and sent to the API.
        :raises: RequestError if req.ok returns false.
        :returns: Dict containing the response from NetBox's API.
        """
        headers = {
            'Content-Type': 'application/json; version={};'.format(
                self.version or self.get_version()
            ),
            'authorization': 'Token {}'.format(self.token),
        }
        if self.session_key:
            headers.update(
                {'X-Session-Key': self.session_key}
            )
        req = requests.put(self.url,
                           headers=headers,
                           data=json.dumps(data),
                           verify=self.ssl_verify)
        if req.ok:
            return req.json()
        else:
            raise RequestError(req)

    def post(self, data):
        """Makes POST request.

        Makes a POST request to NetBox's API. Adds the session key to
        headers if the `private_key` attribute was populated.

        :param data: (dict) Contains a dict that will be turned into a
            json object and sent to the API.
        :raises: RequestError if req.ok returns false.
        :Returns: Dict containing the response from NetBox's API.
        """
        headers = {
            'Content-Type': 'application/json; version={};'.format(
                self.version or self.get_version()
            ),
            'authorization': 'Token {}'.format(self.token),
        }
        if self.session_key:
            headers.update(
                {'X-Session-Key': self.session_key}
            )
        req = requests.post(
            self.normalize_url(self.url),
            headers=headers,
            data=json.dumps(data),
            verify=self.ssl_verify
        )
        if req.ok:
            return req.json()
        else:
            raise RequestError(req)

    def delete(self):
        """Makes DELETE request.

        Makes a DELETE request to NetBox's API. Adds the session key to
        headers if the `private_key` attribute was populated.

        Returns:
            True if successful.

        Raises:
            RequestError if req.ok doesn't return True.
        """
        headers = {
            'accept': 'application/json; version={};'.format(
                self.version or self.get_version()
            ),
            'authorization': 'Token {}'.format(self.token),
        }
        req = requests.delete(
            "{}".format(self.url),
            headers=headers,
            verify=self.ssl_verify
        )
        if req.ok:
            return True
        else:
            raise RequestError(req)



from pkg_resources import get_distribution, DistributionNotFound

from pynetbox.api import Api as api
from pynetbox.lib import RequestError

try:
    __version__ = get_distribution(__name__).version
except DistributionNotFound:
    pass



import unittest
import six

from .util import Response
import pynetbox

if six.PY3:
    from unittest.mock import patch
else:
    from mock import patch


api = pynetbox.api(
    "http://localhost:8000",
    version='2.0'
)

nb = api.virtualization

HEADERS = {
    'accept': 'application/json; version=2.0;'
}


class GenericTest(object):
    name = None
    ret = pynetbox.lib.response.Record
    app = 'virtualization'

    def test_get_all(self):
        with patch(
            'pynetbox.lib.query.requests.get',
            return_value=Response(fixture='{}/{}.json'.format(
                self.app,
                self.name
            ))
        ) as mock:
            ret = getattr(nb, self.name).all()
            self.assertTrue(ret)
            self.assertTrue(isinstance(ret, list))
            self.assertTrue(isinstance(ret[0], self.ret))
            mock.assert_called_with(
                'http://localhost:8000/api/{}/{}/'.format(
                    self.app,
                    self.name.replace('_', '-')
                ),
                headers=HEADERS,
                verify=True
            )

    def test_filter(self):
        with patch(
            'pynetbox.lib.query.requests.get',
            return_value=Response(fixture='{}/{}.json'.format(
                self.app,
                self.name
            ))
        ) as mock:
            ret = getattr(nb, self.name).filter(pk=1)
            self.assertTrue(ret)
            self.assertTrue(isinstance(ret, list))
            self.assertTrue(isinstance(ret[0], self.ret))
            mock.assert_called_with(
                'http://localhost:8000/api/{}/{}/?pk=1'.format(
                    self.app,
                    self.name.replace('_', '-')
                ),
                headers=HEADERS,
                verify=True
            )

    def test_get(self):
        with patch(
            'pynetbox.lib.query.requests.get',
            return_value=Response(fixture='{}/{}.json'.format(
                self.app,
                self.name[:-1]
            ))
        ) as mock:
            ret = getattr(nb, self.name).get(1)
            self.assertTrue(ret)
            self.assertTrue(isinstance(ret, self.ret))
            mock.assert_called_with(
                'http://localhost:8000/api/{}/{}/1/'.format(
                    self.app,
                    self.name.replace('_', '-')
                ),
                headers=HEADERS,
                verify=True
            )


class ClusterTypesTestCase(unittest.TestCase, GenericTest):
    name = 'cluster_types'


class ClusterGroupsTestCase(unittest.TestCase, GenericTest):
    name = 'cluster_groups'


class ClustersTestCase(unittest.TestCase, GenericTest):
    name = 'clusters'


class VirtualMachinesTestCase(unittest.TestCase, GenericTest):
    name = 'virtual_machines'


class InterfacesTestCase(unittest.TestCase, GenericTest):
    name = 'interfaces'



from setuptools import setup

setup(
    name='pynetbox',
    description='NetBox API client library',
    url='https://github.com/digitalocean/pynetbox',
    author='Zach Moody',
    author_email='zmoody@do.co',
    license='Apache2',
    include_package_data=True,
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    packages=[
        'pynetbox',
        'pynetbox.lib'
    ],
    install_requires=[
        'netaddr==0.*',
        'requests==2.*',
        'six==1.*',
    ],
    zip_safe=False,
    keywords=['netbox'],
    classifiers=[
        'Intended Audience :: Developers',
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],

)
